@@ -14,6 +14,12 @@ from .hooks import HookFinder, get_global_hook_path
 from .vendor import yaml
 
 
+__all__ = [
+    'Module',
+    'ModuleSpec',
+]
+
+
 module_header = '''
 # Variables
 # $MODULE - path to this module
@@ -37,7 +43,11 @@ class Module(object):
     def __init__(self, path, name=None, version=None, repo=None):
 
         self.path = paths.normalize(path)
-        self.repo = repo
+        if repo is None:
+            from . import repos
+            self.repo = repos.LocalRepo('_tmp', paths.parent(self.path))
+        else:
+            self.repo = repo
 
         # Create HookFinder for this module
         self.hook_path = self.relative_path('hooks')
@@ -55,7 +65,7 @@ class Module(object):
         }
         self._raw_config = None
         self._config = None
-        self._environ = None
+        self._env = None
 
         # Determine name, version, qual_name, and real_name
 
@@ -106,18 +116,10 @@ class Module(object):
             self.version,
         )
 
-    def as_spec(self):
-        return ModuleSpec(
-            name=self.name,
-            real_name=self.real_name,
-            qual_name=self.qual_name,
-            version=self.version,
-            path=self.path,
-            repo=self.repo,
-        )
-
     @classmethod
     def from_spec(cls, module_spec):
+        '''Create a module from a ModuleSpec object.'''
+
         return cls(
             name=module_spec.name,
             version=module_spec.version.string,
@@ -125,6 +127,18 @@ class Module(object):
             repo=module_spec.repo,
         )
 
+    def to_spec(self, **kwargs):
+        '''Return a ModuleSpec object for this Module.'''
+
+        return ModuleSpec(
+            name=kwargs.get('name', self.name),
+            path=kwargs.get('path', self.path),
+            qual_name=kwargs.get('qual_name', self.qual_name),
+            real_name=kwargs.get('real_name', self.real_name),
+            repo=kwargs.get('repo', self.repo),
+            version=kwargs.get('version', self.version),
+        )
+
     def relative_path(self, *args):
         '''Get a path relative to this module'''
 
@@ -137,18 +151,6 @@ class Module(object):
         if hook:
             return hook.run(self)
 
-    def spec(self, **kwargs):
-        '''Return a ModuleSpec object for this Module.'''
-
-        return ModuleSpec(
-            name=kwargs.get('name', self.name),
-            real_name=kwargs.get('real_name', self.real_name),
-            qual_name=kwargs.get('qual_name', self.qual_name),
-            path=kwargs.get('path', self.path),
-            version=kwargs.get('version', self.version),
-            repo=kwargs.get('repo', None),
-        )
-
     def activate(self):
         '''Add this module to active modules'''
 
@@ -166,25 +168,6 @@ class Module(object):
         api.remove_active_module(self)
         self.run_hook('post_remove')
 
-    @property
-    def icon(self):
-        return self.relative_path('icon.png')
-
-    def has_icon(self):
-        return os.path.isfile(self.icon)
-
-    @property
-    def description(self):
-        return self.config.get('description', '')
-
-    @property
-    def author(self):
-        return self.config.get('author', '')
-
-    @property
-    def email(self):
-        return self.config.get('email', '')
-
     @property
     def is_active(self):
         from . import api
@@ -222,16 +205,36 @@ class Module(object):
         return self._config
 
     @property
-    def environment(self):
-        if self._environ is None:
-            env = self.config.get('environment', {})
-            additional = {
-                'CPENV_ACTIVE_MODULES': [self.real_name],
-            }
-            env = mappings.join_dicts(additional, env)
-            self._environ = mappings.preprocess_dict(env)
+    def author(self):
+        return self.config.get('author', '')
 
-        return self._environ
+    @property
+    def description(self):
+        return self.config.get('description', '')
+
+    @property
+    def email(self):
+        return self.config.get('email', '')
+
+    @property
+    def environment(self):
+        if self._env is None:
+            self._env = self.config.get('environment', {})
+            self._env['CPENV_ACTIVE_MODULES'] = {'append': self.real_name}
+
+        return self._env
+
+    @property
+    def requires(self):
+        return self.config.get('email', [])
+
+    @property
+    def icon(self):
+        return self.relative_path('icon.png')
+
+    @property
+    def has_icon(self):
+        return os.path.isfile(self.icon)
 
 
 def read_raw_config(module_file):
@@ -246,7 +249,7 @@ def read_config(module_file, config_vars=None, data=None):
 
     if config_vars is None:
         config_vars = {
-            'MODULE': paths.normalize(os.path.dirname(module_file)),
+            'MODULE': paths.parent(module_file),
             'PLATFORM': compat.platform,
             'PYVER': sys.version[:3],
         }
@@ -326,11 +329,12 @@ def is_exact_match(requirement, module_spec):
 
     name, version = parse_module_requirement(requirement)
     return (
-        module_spec.qual_name == requirement or
-        module_spec.real_name == requirement or
-        (
-            version and module_spec.name == name and
-            module_spec.version == version
+        module_spec.qual_name == requirement
+        or module_spec.real_name == requirement
+        or (
+            version
+            and module_spec.name == name
+            and module_spec.version == version
         )
     )
 
